/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#ifndef SDL_FREETYPE_CACHE_H
#define SDL_FREETYPE_CACHE_H

#include  "hash.h"

struct _sdl_freetype_cache;
typedef struct _sdl_freetype_cache sdl_freetype_cache_t;

typedef struct _sdl_freetype_cache_entry {
    sdl_freetype_hash_entry_t base;
    unsigned long size;
} sdl_freetype_cache_entry_t;

typedef sdl_freetype_hash_ops_t sdl_freetype_cache_ops_t;

typedef void
(* sdl_freetype_cache_callback_func_t) (sdl_freetype_cache_t * cache,
					sdl_freetype_cache_entry_t * entry,
					void * closure);

sdl_freetype_cache_t *
sdl_freetype_cache_create (int maxsize,
			   const sdl_freetype_cache_ops_t * ops);

void
sdl_freetype_cache_shrink (sdl_freetype_cache_t * cache, int size);

void
sdl_freetype_cache_freeze (sdl_freetype_cache_t * cache);

void
sdl_freetype_cache_thaw (sdl_freetype_cache_t * cache);

sdl_freetype_cache_entry_t *
sdl_freetype_cache_lookup (sdl_freetype_cache_t * cache,
			   sdl_freetype_cache_entry_t * key);

int
sdl_freetype_cache_insert (sdl_freetype_cache_t * cache,
			   sdl_freetype_cache_entry_t * entry);

int
sdl_freetype_cache_foreach (sdl_freetype_cache_t * cache,
			    sdl_freetype_cache_callback_func_t func,
			    void * closure);

void
sdl_freetype_cache_empty (sdl_freetype_cache_t * cache);

void
sdl_freetype_cache_destroy (sdl_freetype_cache_t * cache);

#endif
