/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#ifndef SDL_FREETYPE_HASH_H
#define SDL_FREETYPE_HASH_H

typedef struct _sdl_freetype_hash sdl_freetype_hash_t;
typedef struct _sdl_freetype_hash_entry {
    void * key;
} sdl_freetype_hash_entry_t;

typedef struct _sdl_freetype_hash_ops {
    unsigned long
    (* hash) (const sdl_freetype_hash_entry_t* key);

    int
    (* equal) (const sdl_freetype_hash_entry_t * key0,
	       const sdl_freetype_hash_entry_t * key1);

    void
    (* destroy) (sdl_freetype_hash_entry_t * entry);
} sdl_freetype_hash_ops_t;

typedef void
(* sdl_freetype_hash_callback_func_t) (sdl_freetype_hash_t * hash,
				       sdl_freetype_hash_entry_t * entry,
				       void * closure);
typedef int
(* sdl_freetype_hash_filter_func_t) (sdl_freetype_hash_t * hash,
				     sdl_freetype_hash_entry_t * entry);

struct _sdl_freetype_hash {
    int size;
    int current_entries;
    int itering;
    sdl_freetype_hash_ops_t ops;
    sdl_freetype_hash_entry_t ** entries;
};

sdl_freetype_hash_t *
sdl_freetype_hash_create (int size,
			  const sdl_freetype_hash_ops_t * ops);

sdl_freetype_hash_t *
sdl_freetype_hash_init (sdl_freetype_hash_t * hash, int size,
			const sdl_freetype_hash_ops_t * ops);

sdl_freetype_hash_entry_t *
sdl_freetype_hash_lookup (sdl_freetype_hash_t * hash,
			  sdl_freetype_hash_entry_t * key);

sdl_freetype_hash_entry_t *
sdl_freetype_hash_any_entry (sdl_freetype_hash_t * hash,
			     sdl_freetype_hash_filter_func_t func);

int
sdl_freetype_hash_insert (sdl_freetype_hash_t * hash,
			  sdl_freetype_hash_entry_t * entry);

int
sdl_freetype_hash_remove (sdl_freetype_hash_t * hash,
			  sdl_freetype_hash_entry_t * key);

int
sdl_freetype_hash_remove_all (sdl_freetype_hash_t * hash);

int
sdl_freetype_hash_foreach (sdl_freetype_hash_t * hash,
			   sdl_freetype_hash_callback_func_t func,
			   void * closure);

int
sdl_freetype_hash_resize (sdl_freetype_hash_t * hash);

void
sdl_freetype_hash_destroy (sdl_freetype_hash_t * hash);

void
sdl_freetype_hash_fini (sdl_freetype_hash_t * hash);

#endif
