/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#ifndef SDL_FREETYPE_FONTMAP_H
#define SDL_FREETYPE_FONTMAP_H

#include "sdl-freetype.h"
#include <ft2build.h>
#include FT_FREETYPE_H

typedef struct {
    int x1, y1, x2, y2;
} sdl_freetype_box_t;

typedef struct _sdl_freetype_font_map sdl_freetype_font_map_t;
typedef struct _sdl_freetype_font_face sdl_freetype_font_face_t;
typedef struct _sdl_freetype_base_font sdl_freetype_base_font_t;

typedef struct {
    sdl_freetype_antialias_t antialias;
    sdl_freetype_subpixel_order_t subpixel_order;
    sdl_freetype_hint_style_t hint;
    int embedded_bitmap;
    int embolden;
    int vertical_layout;
    int autohint;
} sdl_freetype_font_options_t;

struct _sdl_freetype_base_font {
    sdl_freetype_hash_entry_t base;

    int refcnt;

    int from_memory;
    FT_Stream stream;

    int index;

    FT_Face face;

    int selected;
    double size;
    FT_Matrix xform;

    int lock;

    int num_faces;
    sdl_freetype_font_face_t * faces;
};

struct _sdl_freetype_font_face {
    int refcnt;

    sdl_freetype_font_options_t options;
    sdl_freetype_base_font_t * base_font;

    struct _sdl_freetype_font_face * next;
};

void
sdl_freetype_font_map_reset (void);

sdl_freetype_font_map_t *
sdl_freetype_font_map_lock (void);

void
sdl_freetype_font_map_unlock (sdl_freetype_font_map_t * fontmap);

sdl_freetype_base_font_t *
sdl_freetype_base_font_create (const char * file,
                               int index);

sdl_freetype_base_font_t *
sdl_freetype_base_font_create_from_memory (void * data,
                                           off_t size,
                                           int index);

sdl_freetype_base_font_t *
sdl_freetype_base_font_reference (sdl_freetype_base_font_t * base_font);

FT_Face
sdl_freetype_base_font_lock_face (sdl_freetype_base_font_t * base_font);

void
sdl_freetype_base_font_unlock_face (sdl_freetype_base_font_t * base_font);

void
sdl_freetype_base_font_select_size (sdl_freetype_base_font_t * base_font,
                                    double size, FT_Matrix * xform);

FT_Library
sdl_freetype_base_font_get_freetype_library (sdl_freetype_base_font_t * base_font);

void
sdl_freetype_base_font_destroy (sdl_freetype_base_font_t * base_font);

sdl_freetype_font_face_t *
sdl_freetype_font_face_create (sdl_freetype_base_font_t * base_font,
                               const sdl_freetype_font_options_t * options);

sdl_freetype_font_face_t *
sdl_freetype_font_face_reference (sdl_freetype_font_face_t * face);

void
sdl_freetype_font_face_destroy (sdl_freetype_font_face_t * face);

#endif
