/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#ifndef SDL_FREETYPE_H
#define SDL_FREETYPE_H

#include <ft2build.h>
#include FT_FREETYPE_H

#include "sdl-freetype-error.h"

#ifdef __cplusplus
extern "C" {
#endif

struct _sdl_freetype_font;
typedef struct _sdl_freetype_font sdl_freetype_font_t;

typedef FT_Int32 sdl_freetype_fixed_t;
typedef enum {
    SDL_FREETYPE_TRUE  = 1,
    SDL_FREETYPE_FALSE = 0
} sdl_freetype_bool_t;

typedef struct {
    FT_UInt index;
    sdl_freetype_fixed_t  x;
    sdl_freetype_fixed_t  y;
} sdl_freetype_glyph_t;

typedef struct {
    sdl_freetype_fixed_t x1, x2, y1, y2;
} sdl_freetype_bbox_t;

typedef struct {
    double x_bearing;
    double y_bearing;
    double width;
    double height;
    double x_advance;
    double y_advance;
} sdl_freetype_text_extents_t;

typedef struct {
    sdl_freetype_fixed_t x_bearing;
    sdl_freetype_fixed_t y_bearing;
    sdl_freetype_fixed_t width;
    sdl_freetype_fixed_t height;
    sdl_freetype_fixed_t x_advance;
    sdl_freetype_fixed_t y_advance;
} sdl_freetype_text_fixed_extents_t;

typedef struct {
    double ascent;
    double descent;
    double height;
    double max_x_advance;
    double max_y_advance;
} sdl_freetype_font_extents_t;

typedef struct {
    sdl_freetype_fixed_t ascent;
    sdl_freetype_fixed_t descent;
    sdl_freetype_fixed_t height;
    sdl_freetype_fixed_t max_x_advance;
    sdl_freetype_fixed_t max_y_advance;
} sdl_freetype_font_fixed_extents_t;

typedef enum {
    SDL_FREETYPE_GLYPH_NONE           = 0,
    SDL_FREETYPE_GLYPH_EMPTY          = 1L << 0,
    SDL_FREETYPE_GLYPH_METRICS        = 1L << 1,
    SDL_FREETYPE_GLYPH_SURFACE        = 1L << 2,
    SDL_FREETYPE_GLYPH_SURFACE_DATA   = 1L << 3,
    SDL_FREETYPE_GLYPH_OUTLINE        = 1L << 4,
    SDL_FREETYPE_GLYPH_USER_DATA      = 1L << 5,
} sdl_freetype_glyph_flags_t;

typedef struct {
    sdl_freetype_glyph_flags_t flags;
    sdl_freetype_text_fixed_extents_t metrics;
    sdl_freetype_bbox_t bbox;
    FT_Bitmap bitmap;
    int x_offset;
    int y_offset;
    void * data;
} sdl_freetype_glyph_info_t;

typedef enum {
    SDL_FT_ANTIALIAS_DEFAULT,
    SDL_FT_ANTIALIAS_NONE,
    SDL_FT_ANTIALIAS_GRAY,
    SDL_FT_ANTIALIAS_SUBPIXEL
} sdl_freetype_antialias_t;

typedef enum {
    SDL_FT_SUBPIXEL_ORDER_DEFAULT,
    SDL_FT_SUBPIXEL_ORDER_RGB,
    SDL_FT_SUBPIXEL_ORDER_BGR,
    SDL_FT_SUBPIXEL_ORDER_VRGB,
    SDL_FT_SUBPIXEL_ORDER_VBGR
} sdl_freetype_subpixel_order_t;

typedef enum {
    SDL_FT_HINT_STYLE_DEFAULT,
    SDL_FT_HINT_STYLE_NONE,
    SDL_FT_HINT_STYLE_SLIGHT,
    SDL_FT_HINT_STYLE_MEDIUM,
    SDL_FT_HINT_STYLE_FULL
} sdl_freetype_hint_style_t;

typedef enum {
    SDL_FT_RENDER_CAPS_NONE      = 0,
    SDL_FT_RENDER_CAPS_OUTLINE   = 1L << 0,
    SDL_FT_RENDER_CAPS_CLEARTYPE = 1L << 1,
} sdl_freetype_render_caps_t;

typedef struct _sdl_freetype_glyph_render sdl_freetype_glyph_render_t;

struct _sdl_freetype_glyph_render {
    sdl_freetype_render_caps_t caps;

    sdl_freetype_error_t
    (* glyph_user_data_init) (sdl_freetype_font_t * font,
                              sdl_freetype_glyph_render_t * render,
                              sdl_freetype_glyph_info_t * info,
                              FT_UInt index);

    void
    (* glyph_user_data_fini) (sdl_freetype_font_t * font,
                              sdl_freetype_glyph_render_t * render,
                              void * data);

    sdl_freetype_error_t
    (* glyph_render) (sdl_freetype_font_t * font,
                      void * dst,
                      unsigned int r,
                      unsigned int g,
                      unsigned int b,
                      unsigned int a,
                      int x, int y,
                      FT_UInt index,
                      const FT_Bitmap * glyph,
                      int x_offset,
                      int y_offset);

    sdl_freetype_error_t
    (* glyphs_render) (sdl_freetype_font_t * font,
                       void * dst,
                       unsigned int r,
                       unsigned int g,
                       unsigned int b,
                       unsigned int a,
                       int x, int y,
                       const sdl_freetype_glyph_t * glyphs,
                       int num_glyphs);

    void
    (* destroy) (sdl_freetype_font_t * font,
                 sdl_freetype_glyph_render_t * render);
};

#define SDL_FONT_ANTIALIAS_MASK           (1 << 0)
#define SDL_FONT_SUBPIXEL_ORDER_MASK      (1 << 1)
#define SDL_FONT_HINT_STYLE_MASK          (1 << 2)
#define SDL_FONT_EMBOLDEN_MASK            (1 << 3)
#define SDL_FONT_TRANSFORM_MASK           (1 << 4)
#define SDL_FONT_INDEX_MASK               (1 << 5)
#define SDL_FONT_EMBEDDEDBITMAP_MASK      (1 << 6)
#define SDL_FONT_VERTICAL_LAYOUT_MASK     (1 << 7)
#define SDL_FONT_AUTOHINT_MASK            (1 << 8)
#define SDL_FONT_FROM_MEMORY_MASK         (1 << 9)
#define SDL_FONT_GLYPH_RENDER_MASK        (1 << 10)
#define SDL_FONT_STROKE_MASK              (1 << 11)
#define SDL_FONT_STROKE_SCALE_MASK        (1 << 12)

typedef struct {
    sdl_freetype_antialias_t antialias;
    sdl_freetype_subpixel_order_t subpixel_order;
    sdl_freetype_hint_style_t hint;
    sdl_freetype_glyph_render_t * render;
    int index;
    int embedded_bitmap;
    int embolden;
    int vertical_layout;
    int autohint;
    FT_Matrix xform;
    double stroke_scale;
    void * data;
    off_t size;
    int stroke;
} sdl_freetype_font_desc_t;

sdl_freetype_error_t
sdl_freetype_init (void);

void
sdl_freetype_fini (void);

const char *
sdl_freetype_version_string (void);

int
sdl_freetype_verion (void);

sdl_freetype_font_t *
sdl_freetype_font_create (const char * file, double size);

sdl_freetype_font_t *
sdl_freetype_font_create_full (const char * file,
                               double size,
                               const sdl_freetype_font_desc_t * desc,
                               unsigned int mask);

sdl_freetype_font_t *
sdl_freetype_font_reference (sdl_freetype_font_t * font);

void
sdl_freetype_font_destroy (sdl_freetype_font_t * font);

FT_Face
sdl_freetype_font_lock_face (sdl_freetype_font_t * font);

void
sdl_freetype_font_unlock_face (sdl_freetype_font_t * font);

sdl_freetype_error_t
sdl_freetype_font_set_render (sdl_freetype_font_t * font,
                              sdl_freetype_glyph_render_t * render);

sdl_freetype_glyph_render_t *
sdl_freetype_font_get_render (sdl_freetype_font_t * font);

sdl_freetype_glyph_info_t *
sdl_freetype_font_glyph_info_lookup (sdl_freetype_font_t * font, FT_UInt index,
                                     sdl_freetype_glyph_flags_t what);

FT_UInt
sdl_freetype_font_glyph_index (sdl_freetype_font_t * font,
                               FT_Int32 ucs4);

sdl_freetype_error_t
sdl_freetype_font_glyph_metrics (sdl_freetype_font_t * font, FT_UInt index,
                                 sdl_freetype_text_extents_t * metrics);

sdl_freetype_error_t
sdl_freetype_font_glyph_fixed_metrics (sdl_freetype_font_t * font, FT_UInt index,
                                       sdl_freetype_text_fixed_extents_t * metrics);

sdl_freetype_error_t
sdl_freetype_font_glyph_surface (sdl_freetype_font_t * font, FT_UInt index,
                                 FT_Bitmap ** surface, int * x_offset,
                                 int * y_offset, void ** data);

sdl_freetype_error_t
sdl_freetype_font_glyph_free_surface_data (sdl_freetype_font_t * font, FT_UInt index,
                                           sdl_freetype_glyph_info_t * info);

sdl_freetype_error_t
sdl_freetype_font_glyph_get_user_data (sdl_freetype_font_t * font, FT_UInt index,
                                       void ** data);

sdl_freetype_error_t
sdl_freetype_font_glyph_clear_user_data (sdl_freetype_font_t * font, FT_UInt index);

void
sdl_freetype_font_glyph_freeze (sdl_freetype_font_t * font);

void
sdl_freetype_font_glyph_thaw (sdl_freetype_font_t * font);

sdl_freetype_error_t
sdl_freetype_font_extents (sdl_freetype_font_t * font,
                           sdl_freetype_font_extents_t * extents);

sdl_freetype_error_t
sdl_freetype_font_fixed_extents (sdl_freetype_font_t * font,
                                 sdl_freetype_font_fixed_extents_t * extents);

sdl_freetype_error_t
sdl_freetype_font_show_text (sdl_freetype_font_t * font, void * dst,
                             unsigned char r, unsigned char g, unsigned char b,
                             unsigned char a, int x, int y, const FT_Int32 * ucs4,
                             int len);

sdl_freetype_error_t
sdl_freetype_font_glyphs_extents (sdl_freetype_font_t * font,
                                  sdl_freetype_text_extents_t * extents,
                                  const sdl_freetype_glyph_t * glyphs,
                                  int num_glyphs);

sdl_freetype_error_t
sdl_freetype_font_glyphs_fixed_extents (sdl_freetype_font_t * font,
                                        sdl_freetype_text_fixed_extents_t * extents,
                                        const sdl_freetype_glyph_t * glyphs,
                                        int num_glyphs);

sdl_freetype_error_t
sdl_freetype_font_text_extents (sdl_freetype_font_t * font,
                                sdl_freetype_text_extents_t * extents,
                                const FT_Int32 * ucs4, int len);

sdl_freetype_error_t
sdl_freetype_font_text_fixed_extents (sdl_freetype_font_t * font,
                                      sdl_freetype_text_fixed_extents_t * extents,
                                      const FT_Int32 * ucs4, int len);

sdl_freetype_error_t
sdl_freetype_font_show_glyphs (sdl_freetype_font_t * font, void * dst,
                               unsigned char r, unsigned char g, unsigned char b,
                               unsigned char a, int x, int y,
                               const sdl_freetype_glyph_t * glyphs, int num_glyphs);

sdl_freetype_error_t
sdl_freetype_font_show_utf8 (sdl_freetype_font_t * font,
                             void * dst,
                             unsigned char r,
                             unsigned char g,
                             unsigned char b,
                             unsigned char a, 
                             int x, int y, 
                             const char * utf8, int len);

sdl_freetype_error_t
sdl_freetype_font_utf8_extents (sdl_freetype_font_t * font,
                                sdl_freetype_text_extents_t * extents,
                                const char * utf8, int len);

sdl_freetype_error_t
sdl_freetype_font_utf8_fixed_extents (sdl_freetype_font_t * font,
                                      sdl_freetype_text_fixed_extents_t * extents,
                                      const char * utf8, int len);

sdl_freetype_error_t
sdl_freetype_font_show_utf16 (sdl_freetype_font_t * font,
			      void * dst,
			      unsigned char r,
			      unsigned char g,
			      unsigned char b,
			      unsigned char a,
			      int x, int y,
			      const short * utf16, int len);

sdl_freetype_error_t
sdl_freetype_font_utf16_extents (sdl_freetype_font_t * font,
                                sdl_freetype_text_extents_t * extents,
                                const short * utf16, int len);

sdl_freetype_error_t
sdl_freetype_font_utf16_fixed_extents (sdl_freetype_font_t * font,
				       sdl_freetype_text_fixed_extents_t * extents,
				       const short * utf16, int len);

#ifdef __cplusplus
}
#endif

#endif

