/* Copyright (C) <2011> Luo Jinghua<sunmoon1997@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include "sdl-freetype.h"
#include "sdl-ft.h"

#ifndef MAX
#define MAX(a, b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif

/* Call this instead of exit(), so we can clean up SDL: atexit() is evil. */
static void
quit(int rc)
{
    SDL_Quit();
    exit(rc);
}

static void
SDL_RectIntersect (SDL_Rect * dest, SDL_Rect * src)
{
    int x1, y1, x2, y2;

    x1 = MAX (dest->x, src->x);
    y1 = MAX (dest->y, src->y);
    x2 = MIN (dest->x + dest->w, src->x + src->w);
    y2 = MIN (dest->y + dest->h, src->y + src->h);

    if (x1 >= x2 || y1 >= y2) {
        dest->x = 0;
        dest->y = 0;
        dest->w = 0;
        dest->h = 0;
    } else {
        dest->x = x1;
        dest->y = y1;
        dest->w = x2 - x1;
        dest->h = y2 - y1;
    }
}

#define FONT "/usr/share/fonts/ttf-bitstream-vera/Vera.ttf"
#define SIZE 32

#define UTF8_TEXT "sdl-freetype-test"
#define UTF8_TEXTLEN -1

int
main(int argc, char *argv[])
{
    SDL_Surface *screen;
    sdl_freetype_font_desc_t desc;
    sdl_freetype_font_t * font;
    int mask;

    Uint8 video_bpp;
    Uint32 videoflags, color;
    int done;
    SDL_Event event;

    /* Initialize SDL */
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
        return (1);
    }

    videoflags = SDL_SWSURFACE;
    video_bpp = 32;

    /* Set 640x480 video mode */
    if ((screen = SDL_SetVideoMode(640, 480, video_bpp, videoflags)) == NULL) {
        fprintf(stderr, "Couldn't set 640x480x%d video mode: %s\n",
                video_bpp, SDL_GetError());
        quit(2);
    }

#if 1
    desc.antialias = SDL_FT_ANTIALIAS_SUBPIXEL;
    desc.vertical_layout = 1;
    desc.subpixel_order = SDL_FT_SUBPIXEL_ORDER_RGB;
    desc.stroke = 1;
    desc.stroke_scale = 1.1;
    mask = 0;
    //mask |= SDL_FONT_ANTIALIAS_MASK;
    mask |= SDL_FONT_SUBPIXEL_ORDER_MASK;
    //mask |= SDL_FONT_VERTICAL_LAYOUT_MASK;
    mask |= SDL_FONT_STROKE_MASK;
    mask |= SDL_FONT_STROKE_SCALE_MASK;
    font = sdl_freetype_font_create_full (FONT, SIZE, &desc, mask);
#else
    font = sdl_ft_create_font ("sans", 18,
                               SDL_FT_WEIGHT_NORMAL,
                               SDL_FT_SLANT_NORMAL,
                               96);
#endif
    if (!font) {
        fprintf (stderr, "create font failed\n");
        quit(3);
    }

    color = SDL_MapRGBA(screen->format, 255, 255, 255, 255);
    SDL_FillRect (screen, NULL, color);
    SDL_UpdateRect (screen, 0, 0, 0, 0);

    /* Wait for a keystroke */
    done = 0;
    while (!done) {
        /* Check for events */
        while (!done && SDL_WaitEvent(&event)) {
            switch (event.type) {
            case SDL_MOUSEBUTTONDOWN:
            {
                sdl_freetype_text_extents_t extents;
                SDL_Rect screenrect, rect;
                int x, y;

                x = event.button.x;
                y = event.button.y;

                sdl_freetype_font_utf8_extents (font, &extents,
                                                UTF8_TEXT, UTF8_TEXTLEN);
                sdl_freetype_font_show_utf8 (font, screen, 0, 0, 0, 255,
                                             x, y, UTF8_TEXT, UTF8_TEXTLEN);
                rect.x = x + extents.x_bearing;
                rect.y = y + extents.y_bearing;
                rect.w = extents.width;
                rect.h = extents.height;
                screenrect.x = 0;
                screenrect.y = 0;
                screenrect.w = screen->w;
                screenrect.h = screen->h;
                SDL_RectIntersect(&rect, &screenrect);
                SDL_UpdateRects(screen, 1, &rect);
            }
            break;
            case SDL_KEYDOWN:
                /* Any key press quits the app... */
                done = 1;
                break;
            case SDL_QUIT:
                done = 1;
                break;
            default:
                break;
            }
        }
    }
 
    sdl_freetype_font_destroy (font);
    sdl_freetype_fini ();

    SDL_Quit();
    return (0);
}
